package com.gitlink.softbot.utils;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.gitlink.softbot.global.vo.Response;
import com.gitlink.softbot.utils.response.UserResponse;
import com.gitlink.softbot.vo.Webhook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

@Component
public class GitLinkApi {

    @Value("${gitlink.url}")
    private  String URL;

    @Value("${gitlink.token}")
    private String TOKEN;

    @Autowired
    private RestTemplate restTemplate;

    private Map<String, String> headParams;

    public Map<String, String> getHeader(){
        Map<String, String> headParams = new HashMap<>();
        headParams.put("Authorization", "Bearer "+TOKEN);
        return headParams;
    }

    /**
     * 调用平台接口，获取用户项目列表
     *
     * @param uid                用户uid
     * @param params             用户login
     * @param category           需传入created，默认为全部项目， join: 参与项目, created: 创建项目, manage: 管理项目, watched: 关注项目, forked: 复刻项目
     * @return Response          统一响应对象
     */
    public Response getProjectsByLogin(Integer uid, Object[] params, String category){
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", uid.toString());
        if(category!=null && !Objects.isNull(category)){
            inputParams.put("category", category);
        }
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.GET, URL+"/api/v1/", params, "/projects.json", inputParams, null ,getHeader());
        return response;
    }

    public Response getUserByUid(Integer uid){
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", uid.toString());
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.GET, URL+"/api", null, "/users/get_user_info.json", inputParams, null ,getHeader());
        return response;
    }

    public String getUserNameByUid(String uid){
        if(uid!=null){
            Map<String, String> inputParams = new HashMap<>();
            inputParams.put("uid", uid.toString());
            Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.GET, URL+"/api", null, "/users/get_user_info.json", inputParams, null ,getHeader());
            if("ok".equals(response.getCode())){
                return JSON.parseObject(response.getData().toString(), UserResponse.class).getUsername();
            }
        }
        return null;
    }

    public Response getWebhookById(Integer uid, Object[] params,Integer webhookId) throws JsonProcessingException {
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", uid.toString());
        ObjectMapper mapper = new ObjectMapper();
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.GET, URL+"/api/v1", params, "/webhooks/"+webhookId+".json", inputParams, null,getHeader());
        return response;
    }

    public Response addWebhook(Integer uid, Object[] params, Webhook webhook) throws JsonProcessingException {
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", uid.toString());
        ObjectMapper mapper = new ObjectMapper();
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.POST, URL+"/api/v1", params, "/webhooks.json", inputParams, mapper.writeValueAsString(webhook) ,getHeader());
        return response;
    }

    public Response updateWebhook(Integer uid, Object[] params, Integer webhookId, Webhook webhook) throws JsonProcessingException {
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", uid.toString());
        ObjectMapper mapper = new ObjectMapper();
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.PATCH, URL+"/api/v1", params, "/webhooks/"+webhookId+".json", inputParams, mapper.writeValueAsString(webhook) ,getHeader());
        return response;
    }

    public Response deleteWebhook(Integer uid, Object[] params, Integer webhookId) {
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", uid.toString());
        ObjectMapper mapper = new ObjectMapper();
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.DELETE, URL+"/api/v1", params, "/webhooks/"+webhookId+".json", inputParams,null, getHeader());
        return response;
    }

    /**
     * 调用平台中与bot 身份认证相关的接口，auth_active, update_secret, update_private_key
     *
     * @param uid               bot拥有者用户的uid
     * @param botId             bot主键id
     * @param suffix            方法类别 auth_active, update_secret, update_private_key
     * @return Response          统一响应对象
     */
    public Response activeBotAuth(Integer uid, Integer botId, String suffix){
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", uid.toString());
        Object[] params = new Object[]{botId.toString()};
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.POST, URL+"/app", params, "/"+suffix, inputParams, null ,getHeader());
        return response;
    }

    public Response updateCallbackUrl(Integer uid, Integer botId) {
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", uid.toString());
        Object[] params = new Object[]{botId.toString()};
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.POST, URL+"/app", params, "/update_callback_url" , inputParams, null ,getHeader());
        return response;
    }

//    public Response activeBotAuth(Integer uid, Integer botId){
//        Map<String, String> inputParams = new HashMap<>();
//        inputParams.put("uid", uid.toString());
//        Object[] params = new Object[]{botId.toString()};
//        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.POST, URL+"/app", params, "/auth_active", inputParams, null ,getHeader());
//        return response;
//    }
//
//    public Response updateSecet(Integer uid, Integer botId){
//        Map<String, String> inputParams = new HashMap<>();
//        inputParams.put("uid", uid.toString());
//        Object[] params = new Object[]{botId.toString()};
//        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.POST, URL+"/app", params, "/update_secret", inputParams, null ,getHeader());
//        return response;
//    }
//
//    public Response updatePrivateKey(Integer uid, Integer botId){
//        Map<String, String> inputParams = new HashMap<>();
//        inputParams.put("uid", uid.toString());
//        Object[] params = new Object[]{botId.toString()};
//        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.POST, URL+"/app", params, "/update_private_key", inputParams, null ,getHeader());
//        return response;
//    }


}
