package com.gitlink.softbot.utils;


import com.gitlink.softbot.global.vo.Response;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.*;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import java.net.URI;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.Objects;


public class RestTemplateUtil {

    private static Logger logger = (Logger) LoggerFactory.getLogger(RestTemplateUtil.class);
    /**
     * @Description 私有构造函数
     */
    private RestTemplateUtil() {
    }

    /**
     * @Description 获取自定义header
     */
    private static HttpHeaders getCustomHeaders(Map<String, String> headerParams, MediaType mediaType) {
        HttpHeaders headers = new HttpHeaders();
        headers.setAll(headerParams);
        if (mediaType != null) {
            headers.setContentType(mediaType);
        }
        return headers;
    }

    /**
     * @Description 处理调用结果
     */
    private static Response handleResult(ResponseEntity<String> resp) {
        if (resp.getStatusCodeValue() == 200) {
            String body = resp.getBody();
            return Response.ok().data(body);
        }
        return Response.error("调用异常");
    }

    /**
     * httpRequest 发起HTTP请求
     * inputParams必须为String
     *
     * @param customRestTemplate 调用RestTemplate
     * @param method             HttpMethod方法类型
     * @param url                请求URL前缀
     * @param pathParams         请求Path数据
     * @param suffix             请求URL后缀
     * @param inputParams        请求Params数据
     * @param jsonParams         请求Json数据
     * @param headerParams       头部参数
     * @return Response          统一响应对象
     */
    public static Response httpRequest(RestTemplate customRestTemplate, HttpMethod method, String url,Object[] pathParams, String suffix, Map<String, String> inputParams, String jsonParams, Map<String, String> headerParams){
        HttpHeaders customHeaders;
        if(Objects.isNull(jsonParams)){
            customHeaders = getCustomHeaders(headerParams, null);
        } else{
            customHeaders = getCustomHeaders(headerParams, new MediaType("application", "json", StandardCharsets.UTF_8));
        }
        HttpEntity entity = new HttpEntity<>(jsonParams, customHeaders);
        // url拼接 pathParams
        if(!Objects.isNull(pathParams)){
            for (Object pathParam : pathParams) {
                url = url + "/" + pathParam;
            }
        }
        // url拼接 suffix
        if(!Objects.isNull(suffix)){
            url = url + suffix;
        }
        // uri拼接 inputParams
        MultiValueMap<String, String> params = new LinkedMultiValueMap<>();
        params.setAll(inputParams);
        UriComponentsBuilder builder = UriComponentsBuilder.fromHttpUrl(url);
        URI uri = builder.queryParams(params).build().encode().toUri();
        logger.info("远程接口调用地址： " + url);
        logger.info("entity: " + entity);
        ResponseEntity<String> resp = customRestTemplate.exchange(uri, method, entity, String.class);
        return handleResult(resp);
    }


}
